package main

/*
	this parses the command line arguements

	this enables command line options from other packages like 'gui' and 'log'
*/

type args struct {
	Repo      string `arg:"positional"                   help:"go import path"`
	NoWork    bool   `arg:"--no-work"    default:"true"  help:"do not make or modify the go.work file"`
	GoSrc     bool   `arg:"--go-src"     default:"true"  help:"only work in ~/go/src"`
	DryRun    bool   `arg:"--dry-run"                    help:"show what would be run"`
	Recursive bool   `arg:"--recursive"  default:"false" help:"resursively clone all dependencies"`
}

func (a args) Description() string {
	return `
By default, go-clone will find your go.work file and work from there.
Otherwise, it will create a work/ directory.

This will recursively clone the sources for this app into a work/ directory:
	go-clone --recursive go.wit.com/apps/go-clone

This will recursively clone the sources 'the old way' into ~/go/src:
	go-clone --recursive --go-src --no-work go.wit.com/apps/go-clone
`
}

func (args) Version() string {
	return "go-clone " + Version
}
